@@ -22,12 +22,24 @@ LOW_QUALITY = 60
 THUMBNAIL_QUALITY = 30
 
 IMTYPE_DEFAULT = cv2.IMREAD_ANYCOLOR
-INTER_DEFAULT = cv2.INTER_AREA
+INTER_DEFAULT = cv2.INTER_LINEAR
 
 ENCODE_JPEG = ".jpg"
 ENCODE_PNG = ".png"
 
+FLIP_VERTICAL = 0
+FLIP_HORIZONTAL = 1
+FLIP_BOTH = 2
+
 __all__ = ['THUMBNAIL_RESOLUTION', 'SD_RESOLUTION', 'SVGA_RESOLUTION', 'XGA_RESOLUTION', 'WXGA_RESOLUTION',
-           'HD_RESOLUTION', 'HDP_RESOLUTION', 'FHD_RESOLUTION', 'QHD_RESOLUTION', 'UHD_RESOLUTION', 'PX_RGB', 'PX_BGR',
+           'HD_RESOLUTION', 'HDP_RESOLUTION', 'FHD_RESOLUTION', 'QHD_RESOLUTION', 'UHD_RESOLUTION',
+
+           'PX_RGB', 'PX_BGR',
+
            'HIGH_QUALITY', 'DEFAULT_QUALITY', 'BALANCE_QUALITY', 'MEDIUM_QUALITY', 'LOW_QUALITY', 'THUMBNAIL_QUALITY',
-           'IMTYPE_DEFAULT', 'INTER_DEFAULT', 'ENCODE_JPEG', 'ENCODE_PNG']
+
+           'IMTYPE_DEFAULT', 'INTER_DEFAULT',
+
+           'ENCODE_JPEG', 'ENCODE_PNG',
+
+           'FLIP_HORIZONTAL', 'FLIP_VERTICAL', 'FLIP_BOTH']
